#####################################################################
#
#     TCL GSM library 0.2
#    by Gerhard Reithofer 
# mail: gerhardr@tech-edv.co.at
#
# Techn. EDV Reithofer, Technical software solutions
# http://www.tech-edv.co.at
#
# Information mostly retrieved from:
# 
#     Manual Reference
#      AT Command Set
#    (GSM 07.07, GSM 07.05)
#  for SIEMENS Mobile Phone
#          S25
#    and Derivatives
# found at: http://www.min.at/prinz/s25/s25at.pdf
#
#   Developer's Guide:
#    SMS with the SMS PDU-mode
#  Version 1.2 (30.07.97) by Siemens AG
# found at: http://www.min.at/prinz/s25/smspdu.pdf
#
#     Manual Reference
#          OBEX
#   (Object Exchange Protocol)
# for SIEMENS Mobile Phone
#          S25
# found at: http://www.min.at/prinz/s25/s25obex.pdf
#  ... you will find many, many, many usefult things at the 
#      (M)obile (I)nformation (N)etwork
# site by Richard Prinz Richard.Prinz@min.at
#
# ... and NOBI's real great GSM site 
#              http://www.nobbi.com/
#
#
# WARNING - WARNING - WARNING - WARNING - WARNING - WARNING - WARNING
# Many of these functions are untested!
# Yor will loose information if working with this software components!
# DISCLAIMER
# In no case I will be liable for any loss of information or
# any other misfortune, calamity, disaster ... which may be caused
# using this software or components of it!
# WARNING - WARNING - WARNING - WARNING - WARNING - WARNING - WARNING
#
#
#############################################
# ######################################### #
# #   implemented GSM standard features   # #
# ######################################### #
#############################################
#
# Naming conventions - not fully consistent
#
# q_* ... query value(s)
# r_* ... read values(s)
# s_* ... set, select or write value(s)
#
###########################################
#
# GSM_DEVICE informations
#
############################################

# query battery status
# param.: none
# result: { mode } { state }
# known values:
#  modes 0 - battery
#        1 - power supply
#  state 0,20,40,60,80,100 - siemens
#        0,50,70,90        - nokia
# 
proc q_battery_status {} {
  set at_cmd "AT+CBC"
  set mdmres [ModemCommand $at_cmd]
  set stat [get_result_list "+CBC:" [lindex $mdmres 1]]
  return [get_separated_list $stat]
}

# query signal quality
# param.: none
# result: { threshold } { rxquality }
# known values:
#  threshold 0 .. 99 (see http://www.nobbi.com/atgsm.htm)
#  rxquality siemens 0..31 
#            ericson 0..31
#            nokia   0,8,16,24,31 
# 
proc q_signal_quality {} {
  set at_cmd "AT+CSQ"
  set mdmres [ModemCommand $at_cmd]
  set stat [get_result_list "+CSQ:" [lindex $mdmres 1]]
  return [get_separated_list $stat]
}
  
#
# query manufacturer name
# param.: none
# result: manufacturer
proc q_manufacturer {} {
  set at_cmd "AT+CGMI"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
} 

#
# query model information
# param.: none
# result: modelname
proc q_model_info {} {
  set at_cmd "AT+CGMM"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
} 

#
# query revision information
# param.: none
# result: revision
proc q_revision_info {} {
  set at_cmd "AT+CGMR"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
} 

#
# query serial number
# param.: none
# result: serialnumber
proc q_serial_number {} {
  set at_cmd "AT+CGSN"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
} 

#
# query IMSI number of SIM
# param.: none
# result: IMSI
proc q_sim_imsi {} {
  set at_cmd "AT+CIMI"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
} 

#
# query GSM_DEVICE capabilities
# param.: none
# result: {cap1} [{cap2} [...]]
# caps: ??? - S35i: {+CGSM} {+W} {+FCLASS}
proc q_capabilities {} {
  set at_cmd "AT+GCAP"
  set mdmres [ModemCommand $at_cmd]
  set caps [get_result_list "+GCAP:" [lindex $mdmres 1]]
  return [get_separated_list $caps]
} 

#
# net service functions
#
############################################

#
# query provider list
# param.: none
# result: { state "pro_long1" "pro_short1" "pro_num1" } ... 
# e.g.: {2 "A1" "" "23201"} "" {3 "A tele.ring" "" "23207"} "" ... {3 0 2}
# known states: 0 - unknown, 1 available, 2 actiive, 3 prohibited
#  pro_short is always "" (GSM-unsupported)
proc q_provider_list {} {
  set at_cmd "AT+COPS?"
  set mdmres [ModemCommand $at_cmd]
  set slist [get_result_list "+COPS:" [lindex $mdmres 1]]
  return [get_separated_list $slist]
} 

#
# select netservice provider
# param.: mode format net
# result: modem result (OK/ERROR)
#   mode  : 0 - automatic (param. "format" and "net" ignored)
#           1 - manual 
#           3 - leave 
#           4 - try manual (1) and select automaic (0) on fail
#   format: 0 - long format ("A tele.ring")
#           1 - short format (GSM-unsupported)
#           2 - numeric (23207)
# e.g.: 1,0,"A1" 
proc s_provider { mode format net } {
  set at_cmd "AT+COPS="
  append at_cmd $mode
  append at_cmd ","
  append at_cmd $format
  append at_cmd ","
  append at_cmd $net
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
} 

#
# read vibrator status
# param.: none
# result: 0 (off), 1 (on)
proc r_vibrator_state { } { 
  set at_cmd "AT+CVIB?"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}

# select vibrator status
# param.: state (0=off, 1=on)
# result: modem result (OK/ERROR)
proc s_vib_status { state } { 
  set at_cmd "AT+CVIB="
  append $state
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}

############################################
#
# phone book functions
#
############################################

############################################
# query phone books
# param.: none
# result: {book1 [book2 [...]]}
# books: FD (fixed dialing - SIM internal), 
#        LD (last dialled), 
#        ME (mobile equipement - phone internal),
#        MT (combined ME&SM), 
#        OW (own numbers - SIM),
#        SM (SIM memory),
#        TA (??? Numbers in TA)

proc q_phone_books {} {
  set books {}
  set at_cmd "AT+CPBS=?"
  set mdmres [ModemCommand $at_cmd]
  foreach entry [get_result_list "+CPBS:" [lindex $mdmres 1]] {
    ### let's remove the list surrounding "( )"
    set current [string trimleft [string trimright $entry ")"] "("]
    set books [concat $books $current]
  }
  return [get_separated_list $books]
} 

#
# select phone book
# param.: phone book name (see q_phone_books)
# result: modem result (OK/ERROR)
proc s_phone_book { book } {
  set at_cmd "AT+CPBS="
  append at_cmd $book
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}

#
# query phone book capabilities
# param.: none
# result: { book numbers_used max_possible }
proc q_caps_act_pbook {} {
  set at_cmd "AT+CPBS?"
  set mdmres [ModemCommand $at_cmd]
  set caps [get_result_list "+CPBS:" [lindex $mdmres 1]]
  return [get_separated_list $caps]
}

#
# query phone book contents
# param.: phone book name (see q_phone_books)
# result: { min-max numlen namelen }
proc q_pbook_conts { book } {
  set at_cmd "AT+CPBR=?"
  set mdmres [ModemCommand $at_cmd]
  set conts [get_result_list "+CPBR:" [lindex $mdmres 1]]
  return [get_separated_list $conts]
}

#
# read phone book entry
# param.: phone number index
# result: { index number type_of_address name }
# known address types: 145 - internat. number (beginning with "+...")
#                      129 - unknown format
proc r_pbook_entry { index } {
  set at_cmd "AT+CPBR="
  append at_cmd $index
  set mdmres [ModemCommand $at_cmd]
  set number [get_result_list "+CPBR:" [lindex $mdmres 1]]
  return [get_separated_list $number]
}

#
# query phone book format
# param.: none
# result: { min-max number_len typmin-typmax name_len }
#   typmin, typmax (see r_pbook_entry)
proc q_pbook_format {} {
  set at_cmd "AT+CPBW=?"
  append at_cmd $index
  set mdmres [ModemCommand $at_cmd]
  return [get_result_list "+CPBW:" [lindex $mdmres 1]]
}

###########################################################
#
# write phone book entry
# param.: index number type text
# result: modem result (OK/ERROR)
# known address types: 145 - internat. number (beginning with "+...")
#                      129 - unknown format
# bugs  : at the moment we don't check the length's of the entries, 
#         which could be retrieved with 'q_pbook_format'
proc s_pbook_entry { index number type name } {
  set at_cmd "AT+CPBW="
  append at_cmd $index
  append at_cmd ","
  append at_cmd \"$number\"
  append at_cmd ","
  append at_cmd $type
  append at_cmd ","
  append at_cmd \"$name\"
  set mdmres [ModemCommand $at_cmd]
  return [lindex $mdmres 1]
}

